#include <l4/re/c/dataspace.h>

#include "dataspace_server.h"
#include "simple_pager.h"



SimplePager::SimplePager(Memory *memory)
{
    if (memory == NULL)
        _memory = new Memory();
    else
        _memory = memory;

    _region = _memory->region();
}

void SimplePager::close()
{
    if (_region != NULL)
    {
        _memory->release(_region);
        _region = NULL;
    }
}

/* Map a flexpage corresponding to the dataspace 'offset' involving a 'hot_spot'
   (flexpage offset). */

long SimplePager::map(unsigned long offset, l4_addr_t hot_spot, unsigned long flags, l4_snd_fpage_t *region)
{
    Flexpage flexpage(_region);

    flexpage.reset(offset);

    /* Issue the flexpage via the IPC system. */

    SendFlexpage send_flexpage = flexpage.to_send(offset, hot_spot);

    /* Send the flexpage explicitly. */

    region->fpage = l4_fpage(send_flexpage.base_addr, send_flexpage.order,
                             (flags & L4RE_DS_MAP_FLAG_RW) ? L4_FPAGE_RW
                                                           : L4_FPAGE_RO);
    region->snd_base = hot_spot;

    long err = complete_Dataspace_map(*region);

    if (err)
      return err;

    return IPC_MESSAGE_SENT;
}

// vim: tabstop=4 expandtab shiftwidth=4
